#    thumb.py is part of Mediagrid by Luca Franceschini & Luca Carrubba.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import shutil
from subprocess import Popen,PIPE
import platform
import struct

def capture():
    if len(sys.argv) < 6:
	print 'Argomenti insufficenti.'
	sys.exit()
    in_path = os.path.abspath(sys.argv[2])
    out_path = os.path.abspath(sys.argv[3])
    id_mg = sys.argv[4]
    max_mg = int(sys.argv[5])
    sistema = platform.system()
    
    ffmpeg = "ffmpeg"
    if sistema == "Windows":
        ffmpeg = "C:\\ffmpeg\\ffmpeg.exe"
      
    start_mg = lastid(out_path, id_mg)
    if start_mg >= max_mg:
        start_mg = 0

    f = open(os.path.join(out_path, id_mg),'w')

    if sys.argv[1] == "d":
        contatore = 0
        for fileorig in os.listdir(in_path):
            nomefile = os.path.join(in_path, fileorig)
            tipo = file_type(nomefile, sistema)
            if tipo > 0 and contatore < max_mg:
                contatore = contatore + 1
                start_mg = start_mg + 1
                output = "%d%s%s" % (start_mg, " ", nomefile)
                print output     
                gen_thumb(nomefile, out_path, tipo, id_mg, start_mg, ffmpeg)
                f.seek(0)
                f.write(struct.pack('i', start_mg))
                f.truncate()
            if start_mg == max_mg:
                start_mg = 0

    if sys.argv[1] == "f":
        tipo = file_type(in_path, sistema)
        if tipo > 0:
            start_mg = start_mg + 1
            output = "%d%s%s" % (start_mg, " ", in_path)
            print output
            gen_thumb(in_path, out_path, tipo, id_mg, start_mg, ffmpeg)
            f.seek(0)
            f.write(struct.pack('i', start_mg))
            f.truncate()

    f.close()
    return

#funzione che genera le thumb
def gen_thumb(nomefile, out_path, tipo, id_mg, start_mg, ffmpeg):
    newthumb = "%s%s%d%s" % (id_mg, "-", start_mg, ".ppm")
    if tipo == 1:
        middle = get_duration(nomefile, ffmpeg)/2
        try:
            p = Popen([ffmpeg, "-ss", humanize_time(middle), "-vframes", "1", "-i", nomefile, "-vcodec", "ppm", "-f", "image2", "-s", "60x40", os.path.join(out_path, newthumb)], stdout=PIPE, stderr=PIPE)
            p.wait()
        except:
            print "FFMPEG!"
    if tipo == 2:
        try:
            p = Popen([ffmpeg, "-i", nomefile, "-vcodec", "ppm", "-f", "image2", "-s", "60x40", os.path.join(out_path, newthumb)], stdout=PIPE, stderr=PIPE)
            p.wait()
        except:
            print "FFMPEG!"    
    if tipo == 3:
        shutil.copy(os.path.join("default/audio.ppm"), os.path.join(out_path, newthumb))
    return

#funzione che ritorna il tipo di file (1 video, 2 immagine, 3 audio, 0 da scartare)
def file_type(ilfile, sistema):
    import mimetypes
    type = mimetypes.guess_type(ilfile)[0]
    if type is None:
        return 0
    type = type.split("/")[0]
    if type == "audio":
        return 3
    elif type == "image":
        return 2
    elif type == "video":
        return 1
    return 0

#ritorna la durata del video in secondi (-1 se non ha durata)
def get_duration(ilfile, ffmpeg):
    totsec = 2
    try:
        p = Popen([ffmpeg, "-i", ilfile], stdout=PIPE, stderr=PIPE)
        p.wait()
        for line in p.stderr:
            if 'Duration' not in line: continue
            fields = line.split()
            duration = fields[1].replace(',', '').split(".")[0].split(":")
            totsec = int(duration[0])*3600 + int(duration[1])*60 + int(duration[2])
            break
    except:
        print "FFMPEG!"
    return totsec

#passa dai secondi al formato ora:minuti:secondi
def humanize_time(secs):
    mins, secs = divmod(secs, 60)
    hours, mins = divmod(mins, 60)
    return '%02d:%02d:%02d' % (hours, mins, secs)

#funzione che ritorna l'integer dell'ultimo file
def lastid(out_path, id_mg):
    if os.path.isfile(os.path.join(out_path, id_mg)):
      fin = open(os.path.join(out_path, id_mg), "rb")
      try:
        return struct.unpack('i', fin.read(4))[0]
      except:
        return 0
    return 0

if __name__ == '__main__':
    capture()
